unit Main;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, iDragDrop, StdCtrls, ComCtrls, Buttons, ExtCtrls;

type
  TMainForm = class(TForm)
    PageControl1: TPageControl;
    TabMain: TTabSheet;
    TabConfig: TTabSheet;
    Label1: TLabel;
    EditAbout: TEdit;
    Label2: TLabel;
    EditCompile: TEdit;
    BtnCheckAbout: TBitBtn;
    BtnCheckCompile: TBitBtn;
    PnlDropRc: TPanel;
    PnlDropDir: TPanel;
    Label5: TLabel;
    EditRcName: TEdit;
    Label9: TLabel;
    EditInclude: TEdit;
    Label10: TLabel;
    EditIgnoreNameHead: TEdit;
    Label8: TLabel;
    BtnAbout: TBitBtn;
    Label4: TLabel;
    MemTypeID: TMemo;
    Label6: TLabel;
    EditDefaultTypeID: TEdit;
    Label7: TLabel;
    RdoIgnore: TRadioButton;
    RdoNotIgnore: TRadioButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure BtnCheckAboutClick(Sender: TObject);
    procedure BtnCheckCompileClick(Sender: TObject);
    procedure BtnAboutClick(Sender: TObject);
    procedure Label9Click(Sender: TObject);
    procedure Label10Click(Sender: TObject);
    procedure Label5Click(Sender: TObject);
    procedure PnlDropRcClick(Sender: TObject);
    procedure PnlDropDirClick(Sender: TObject);
  private
    Dropper: TShellDrop;
    FStdOut, FErrOut: TStringList;
    FRcOut, FTempSL: TStringList;
    procedure ClearOut();
    procedure ShowOut();
    procedure DroppedAction(AFiles, AFolds: TStringList;
                            AControl: TControl; NotUse02: TPoint);
    procedure ExecuteWithShow(const Line: string);

    function VisitAction(const Path: string; const IsFile: Boolean; var DigDir: Boolean): Boolean;
  public
    procedure RcFileOut(const DirPath: string);
    procedure ResFileOut(const RcPath: string);
  end;

var
  MainForm: TMainForm;

implementation

uses
    GrabConsole, Info;

{$R *.dfm}

//------------------------------------------------------------------------------
procedure InfoOk(const S: string);
var F: integer;
begin
    F := MB_OK + MB_ICONINFORMATION;
    Application.MessageBox(PChar(S), PChar(Application.Title), F);
end;

//------------------------------------------------------------------------------
function QandYesNo(const S: string): Boolean;
var F: integer;
begin
    F := MB_YESNO + MB_ICONQUESTION;
    Result := (Application.MessageBox(PChar(S), PChar(Application.Title), F) = mrYes);
end;

//------------------------------------------------------------------------------
procedure WarnOk(const S: string);
var F: integer;
begin
    F := MB_OK + MB_ICONWARNING;
    Application.MessageBox(PChar(S), PChar(Application.Title), F);
end;

//------------------------------------------------------------------------------
procedure ErrorOk(const S: string);
var F: integer;
begin
    F := MB_OK + MB_ICONERROR;
    Application.MessageBox(PChar(S), PChar(Application.Title), F);
end;

//------------------------------------------------------------------------------
//  t@C/tH_̃R[obN֐
type
    TVisitorProc = function(const Path: string;     //  pX
                            const IsFile: Boolean;  //  t@Cǂ
                            var DigDir: Boolean)    //  tH_̏ꍇ@邩ǂ
                            : Boolean of object;    //  f true

//------------------------------------------------------------------------------
//  ֐
procedure VisitDirectory(const DirPath: string; Proc: TVisitorProc);
var
    Rec: TSearchRec;
    Path: string;
    DigDir, Cancel: Boolean;
begin
    //  \ t
    Path := IncludeTrailingPathDelimiter(DirPath);
    //  񌟍
    if FindFirst(Path  + '*.*', faAnyFile, Rec) = 0 then
    begin
        //  
        try
            repeat
                //  tH_
                if (Rec.Attr and faDirectory > 0) and (Rec.Name <> '.') and (Rec.Name <> '..') then
                begin
                    //  ʒm
                    Cancel := Proc(Path + Rec.Name, false, DigDir);
                    //  LZ̏ꍇ
                    if Cancel then break;
                    //  Ɍ@ꍇ
                    if DigDir then VisitDirectory(Path + Rec.Name, Proc);
                end
                //  t@C
                else
                begin
                    //  ʒm
                    Cancel := Proc(Path + Rec.Name, true, DigDir);
                    //  LZ̏ꍇ
                    if Cancel then break;
                end;
            until
                (FindNext(Rec) <> 0)
            ;
        //  
        finally
            FindClose(Rec);
        end;
    end;
end;

//------------------------------------------------------------------------------
//  RXgN^
procedure TMainForm.FormCreate(Sender: TObject);
begin
    //  LvVݒ
    Self.Caption := Format('%s - %s', [ChangeFileExt(ExtractFileName(ParamStr(0)), ''), Self.Caption]);
    //  R\[pCvpXg쐬
    FStdOut := TStringList.Create;
    FErrOut := TStringList.Create;
    //  ̑쐬
    FRcOut := TStringList.Create;
    FTempSL := TStringList.Create;
    //  VFhbvpNX쐬
    Dropper := TShellDrop.Create(TabMain, DroppedAction);
    Dropper.Start();
end;

//------------------------------------------------------------------------------
//  fXgN^
procedure TMainForm.FormDestroy(Sender: TObject);
begin
    //  VFhbvpNXj
    Dropper.Stop();
    Dropper.Free;
    //  ̑j
    FRcOut.Free;
    FTempSL.Free;
    //  R\[pCvpXgj
    FStdOut.Free;
    FErrOut.Free;
end;

//------------------------------------------------------------------------------
//  R\[pCvpXgNA
procedure   TMainForm.ClearOut();
begin
    FStdOut.Clear;
    FErrOut.Clear;
end;

//------------------------------------------------------------------------------
//  R\[pCvpXge\
procedure   TMainForm.ShowOut();
begin
    //  Wo͂̂ݕ\
    InfoOk(FStdOut.Text);
    //  G[
    if FErrOut.Count > 0 then ErrorOk(FErrOut.Text)
end;

//------------------------------------------------------------------------------
//  sƂʂƂ
procedure   TMainForm.ExecuteWithShow(const Line: string);
begin
    //  NA
    ClearOut();
    //  sāE󂯂Ƃ
    PipeConsole(Line, FStdOut, FErrOut);
    //  \
    ShowOut();
end;

//------------------------------------------------------------------------------
//  t@CtH_Ƃɂǂ邩
function TMainForm.VisitAction(const Path: string; const IsFile: Boolean;
                               var DigDir: Boolean): Boolean;
var
    Ext, Ext0: string;
    Nam, Nam0: string;
    i: integer;
    TypeID: string;
begin
    //  LZ͑ΉȂ̕
    Result := false;

    //  t@C̏ꍇ
    if IsFile then
    begin
        //  gq`FbN
        Ext := LowerCase(ExtractFileExt(Path));
        Ext0 := LowerCase(EditInclude.Text);
        if (AnsiPos(Ext, Ext0) = 0) then Exit;
        //  t@C`FbN
        Nam := ChangeFileExt(ExtractFileName(Path), '');
        FTempSL.CommaText := EditIgnoreNameHead.Text;
        for i := 0 to FTempSL.Count - 1 do
        begin
            Nam0 := FTempSL[i];
            if (Copy(Nam, 1, Length(Nam0)) = Nam0) then Exit;
        end;
        //  ^CvID擾
        TypeID := MemTypeID.Lines.Values[Ext];
        if (TypeID = '') then
        begin
            if RdoIgnore.Checked then Exit;
            TypeID := EditDefaultTypeID.Text;
        end;
        //  o
        Ext := UpperCase(Copy(Ext, 2, Length(Ext)));
        Nam := UpperCase(Nam);
        FRcOut.Add(Format('%s_%s %s %s', [Ext, Nam, TypeID, ExtractFileName(Path)]));
    end
    //  tH_̏ꍇ
    else
        //  ȏ͌@Ȃ
        DigDir := false;
    ;
end;

//------------------------------------------------------------------------------
//  \[XXNvg쐬
procedure TMainForm.RcFileOut(const DirPath: string);
var
    Path: string;
begin
    FRcOut.Clear;
    //  tH_
    VisitDirectory(DirPath, VisitAction);
    if (FRcOut.Count > 0) then
    begin
        Path := IncludeTrailingPathDelimiter(DirPath) + EditRcName.Text;
        if FileExists(Path) then
        begin
            //  ㏑֎~E
            if not QandYesNo(Path + #13#10'͂܂㏑܂H') then
                Exit;
            ;
        end;
        //  .rc 쐬
        FRcOut.SaveToFile(Path);
        //  RpC
        ResFileOut(Path);
    end;
end;

//------------------------------------------------------------------------------
//  RpC
procedure TMainForm.ResFileOut(const RcPath: string);
var
    Command: string;
    ResPath: string;
begin
    ResPath := ChangeFileExt(RcPath, '.res');
    if FileExists(ResPath) then
    begin
        if not QandYesNo(ResPath + #13#10'͂܂㏑܂H') then
            Exit;
        ;
    end;
    Command := Format(EditCompile.Text, [RcPath]);
    ExecuteWithShow(Command);
    if FileExists(ResPath) then
        InfoOk(Format('̓t@CF%s'#13#10'o̓t@CF%s', [RcPath, ResPath]))
    else
        WarnOk(Format('%s'#13#10'RpCł܂łB', [RcPath]));
    ;
end;

//------------------------------------------------------------------------------
//  t@Chbv
procedure   TMainForm.DroppedAction(AFiles, AFolds: TStringList;
                            AControl: TControl; NotUse02: TPoint);
var
    i: integer;
begin
    //  VFhbvt~
    Dropper.Stop();
    //  J[\ύX
    Screen.Cursor := crHourGlass;

    //  Ƃꂽꏊ́H
    if (AControl = PnlDropRc) then
    begin
        //  .rc gq̃t@C(1)
        for i := 0 to AFiles.Count - 1 do
        begin
            if (UpperCase(ExtractFileExt(AFiles[i])) = '.RC') then
            begin
                //  RpC
                ResFileOut(AFiles[i]);
                break;
            end;
        end;
    end
    else if (AControl = PnlDropDir) then
    begin
        //  tH_ƂꂽꍇA1ڂ̃tH_ɂ
        if (AFolds.Count > 0) then
            //  XNvg쐬
            RcFileOut(AFolds[0])
        ;
    end
    else
    begin
        InfoOk('F̃pl̏ɗƂĂB');
    end;

    //  J[\߂
    Screen.Cursor := crDefault;
    //  VFhbvtĊJ
    Dropper.Start();
end;

//------------------------------------------------------------------------------
//  "brcc32 ɂ"R}h̃`FbN
procedure TMainForm.BtnCheckAboutClick(Sender: TObject);
begin
    ExecuteWithShow(EditAbout.Text);
end;

//------------------------------------------------------------------------------
//  "brcc32 ŃRpC"R}h̃`FbN
procedure TMainForm.BtnCheckCompileClick(Sender: TObject);
begin
    ExecuteWithShow(EditCompile.Text);
end;

//------------------------------------------------------------------------------
//  \
procedure TMainForm.BtnAboutClick(Sender: TObject);
begin
    OpenInfoForm();
end;

//------------------------------------------------------------------------------
//  
procedure TMainForm.Label9Click(Sender: TObject);
var
    Msg: string;
begin
    Msg := Format('tH_ɂAgq [ %s ] ̃t@CΏۂɂ܂B', [EditInclude.Text]);
    InfoOk(Msg);
end;

procedure TMainForm.Label10Click(Sender: TObject);
var
    Msg: string;
begin
    Msg := Format('Ώۊgqt@C̓AO [ %s ] ł͂܂t@C͑ΏۊOƂ܂B', [EditIgnoreNameHead.Text]);
    InfoOk(Msg);
end;

procedure TMainForm.Label5Click(Sender: TObject);
var
    Msg: string;
begin
    Msg := Format('tH_hbvꂽꍇA[ %s ] Ƃ̂Ń\[XXNvg'
                  + #13#10'̃tH_ɍ쐬܂B' , [EditRcName.Text]);
    InfoOk(Msg);
end;

procedure TMainForm.PnlDropRcClick(Sender: TObject);
var
    Msg: string;
begin
    Msg := '.rc t@ChbvƁÃtH_ .res t@C쐬܂B';
    InfoOk(Msg);
end;

procedure TMainForm.PnlDropDirClick(Sender: TObject);
var
    Msg: string;
begin
    Msg := 'tH_hbvƁÃtH_ .rc/.res t@C쐬܂B';
    InfoOk(Msg);
end;

end.

